#include "fieldlistwidget.h"
#include <QDebug>
#include <QList>
#include <QHeaderView>
#include <QMessageBox>
#include "common/sqlhelper.h"
FieldListWidget::FieldListWidget(const QString& table,QWidget *parent) :
        QTreeWidget(parent)
{
    mTable = table;
    mRecord = SqlHelper::emptyRecord(table);
    setColumnCount(3);

    header()->stretchLastSection();
    //    header()->resizeSection(3,16);
    header()->setResizeMode(QHeaderView::ResizeToContents);
    setSelectionMode(QAbstractItemView::NoSelection);
    setHeaderHidden(true);
    setRootIsDecorated(false);
    setAlternatingRowColors(true);

}
void FieldListWidget::addField(QString label,QString sqlName, AbstractFieldWidget * field)
{

    QTreeWidgetItem * item =new QTreeWidgetItem;
    QToolButton * button = new QToolButton;

    button->setMaximumSize(32,32);

    addTopLevelItem(item);

    setItemWidget(item,0,button);
    item->setText(1,label);
    item->setTextAlignment(1,Qt::AlignTop);
    item->setData(1,Qt::UserRole,sqlName);
    setItemWidget(item,2,field);
    mFieldWidgets[sqlName] = field;
    mFieldItems[sqlName] = item;
    mFieldButtons[sqlName] = button;

    connect(button,SIGNAL(clicked()),field,SLOT(toogleState()));
    connect(field,SIGNAL(stateChanged()),this,SLOT(stateChanged()));



}
QVariant FieldListWidget::value(QString  sqlName)
{
    if ( mFieldWidgets.contains(sqlName))
        return mFieldWidgets[sqlName]->value();
    else return NULL;
}




void FieldListWidget::setValue(QString sqlName, QVariant value)
{
    if ( mFieldWidgets.contains(sqlName))
        mFieldWidgets[sqlName]->setValue(value);
}


void FieldListWidget::itemClicked(QTreeWidgetItem * item, int col)
{



}
void FieldListWidget::load(QSqlRecord rec)
{
    mRecord = rec;
    for ( int i=0; i<rec.count(); i++)
    {
        QString fieldName = rec.fieldName(i);
        if ( mFieldWidgets.contains(fieldName))  //Si il existe un champs le remplir
        {
            mFieldWidgets[fieldName]->setValue(rec.value(fieldName));
        }
    }

    loadFieldState(rec.value("id").toInt());

}
void FieldListWidget::load(const QString& table, int modelId)
{

    QSqlRecord record = SqlHelper::record(table,modelId);
    load(record);
}

void FieldListWidget::load(const QString& query)
{
    QSqlQuery q(query);
    q.next();
    QSqlRecord record = q.record();
    load(record);
}

QSqlRecord FieldListWidget::record()
{
    for ( int i=0; i<mRecord.count(); i++)
    {
        QString fieldName = mRecord.fieldName(i);
        if ( mFieldWidgets.contains(fieldName))  //Si il existe un champs le remplir
        {
            mRecord.setValue(fieldName,mFieldWidgets[fieldName]->value());
        }
    }

    return mRecord;

}
void FieldListWidget::save()
{
    QSqlRecord rec = record();
    if (!SqlHelper::saveRecord(table(),rec))
    {
        QMessageBox::critical(this,"error","cannot save...");
        return;
    }

    if ( rec.value("id").isNull()) // si pas d' id, alors cest un insert
        saveFieldState(SqlHelper::lastInsertId(table()));


    else
        saveFieldState(rec.value("id").toInt());


}
void FieldListWidget::saveFieldState(int fieldId)
{
    qDebug()<<"save field state";

    QSqlQuery removeQuery;
    removeQuery.exec("DELETE FROM fields WHERE table_name='"+table()+"' AND field_id='"+QString::number(fieldId)+"'");

    QString field;
    QString  state;
    QMapIterator<QString, AbstractFieldWidget*> i(mFieldWidgets);
    while (i.hasNext()) {
        i.next();
        field=i.key();
        state=QString::number(i.value()->state());


        QString requete= "INSERT INTO fields (table_name,field_name,field_id,state)"
                         " VALUES ('"+table()+"','"+field+"','"+QString::number(fieldId)+"','"+state+"')";

        QSqlQuery query;
        if (!query.exec(requete))
            qDebug()<<"Cannot save field state:"+query.lastError().text();
    }



}
void FieldListWidget::loadFieldState(int fieldId)
{
    QString requete= "SELECT * from fields WHERE"
                     " table_name='"+table()+"'"
                     " AND field_id='"+QString::number(fieldId)+"'";
    QSqlQuery query(requete);

    while ( query.next())
    {
        QString fieldName = query.record().value("field_name").toString();
        if ( mFieldWidgets.contains(fieldName))
        {
            mFieldWidgets[fieldName]->setState(query.record().value("state").toInt());
        }

    }
}

void FieldListWidget::stateChanged()
{

    AbstractFieldWidget * field = qobject_cast<AbstractFieldWidget*>(sender());
    QString fieldName = mFieldWidgets.key(field);

    if ( (mFieldButtons.contains(fieldName)) && (mFieldItems.contains(fieldName)))
    {

        QToolButton * button = mFieldButtons[fieldName];
        QTreeWidgetItem * item = mFieldItems[fieldName];
        item->setData(0,Qt::UserRole,field->state());

        switch ( field->state())
        {

        case STATE_ERROR :
            item->setBackgroundColor(0,QColor(255,0,0,50));
            item->setBackgroundColor(1,QColor(255,0,0,50));
            item->setBackgroundColor(2,QColor(255,0,0,50));
            button->setIcon(QIcon(":/icons/small/error.png"));
            break;
        case STATE_VALID :
            item->setBackgroundColor(0,QColor(0,255,0,50));
            item->setBackgroundColor(1,QColor(0,255,0,50));
            item->setBackgroundColor(2,QColor(0,255,0,50));
            button->setIcon(QIcon(":/icons/small/valid.png"));

            break;
        case STATE_EMPTY :
            item->setBackgroundColor(0,QColor(0,0,0,0));
            item->setBackgroundColor(1,QColor(0,0,0,0));
            item->setBackgroundColor(2,QColor(0,0,0,0));
            button->setIcon(QIcon(":/icons/small/empty.png"));

            break;
        case STATE_TODO :
            item->setBackgroundColor(0,QColor(255,255,0,50));
            item->setBackgroundColor(1,QColor(255,255,0,50));
            item->setBackgroundColor(2,QColor(255,255,0,50));
            button->setIcon(QIcon(":/icons/small/todo.png"));

            break;

        }
    }
}

void FieldListWidget::addTitleSpace(QString title)
{

    QTreeWidgetItem * item = new QTreeWidgetItem;
    QFont font;
    font.setBold(true);
    QColor blue ("#2d67b2");
    item->setBackgroundColor(0,blue);
    item->setBackgroundColor(1,blue);
    item->setBackgroundColor(2,blue);

    item->setFont(2,font);
    item->setTextColor(2,Qt::white);
    item->setText(2,title);
    addTopLevelItem(item);
    item->setSizeHint(2, QSize(20,30));


}
void FieldListWidget::filter(const QString& filter)
{

    for (int i=0; i<topLevelItemCount(); i++) {
        QTreeWidgetItem *item = topLevelItem( i );
        item->setHidden( true );
        if (item->text(1).contains(filter)) {
            item->setHidden( false );
        }
    }
}
void FieldListWidget::filterState(int state)
{
    for (int i=0; i<topLevelItemCount(); i++) {
        QTreeWidgetItem *item = topLevelItem( i );
        item->setHidden( true );
        if (item->data(0,Qt::UserRole) == state) {
            item->setHidden( false );
        }
        if (state == -2) {
            item->setHidden( false );
        }

    }



}
