#include "logindialog.h"
#include "ui_logindialog.h"
#include <QtSql>
#include <QMessageBox>
#include <QSettings>
#include <QDebug>
#include "settings/serversettings.h"

LoginDialog::LoginDialog(QWidget *parent) :
        QDialog(parent),
        ui(new Ui::LoginDialog)
{
    ui->setupUi(this);

    connect(ui->buttonBox,SIGNAL(accepted()),this,SLOT(login()));
    connect(ui->buttonSettings,SIGNAL(clicked()),this,SLOT(showServerSettings()));
}

LoginDialog::~LoginDialog()
{
    delete ui;
}
void LoginDialog::login()
{
    QSettings settings;

    QString driver = settings.value("server/driver").toString();
    QString host = settings.value("server/host").toString();
    QString databaseName = settings.value("server/databaseName").toString();

    QString user = ui->lineEditUser->text();
    QString pass = ui->lineEditPass->text();




    while ( driver.isEmpty())
    {
        QMessageBox::warning(this,"setup","you need to setup server");
        showServerSettings();

        QString driver = settings.value("server/driver").toString();
        QString host = settings.value("server/host").toString();
        QString databaseName = settings.value("server/databaseName").toString();
    }




    QSqlDatabase db= QSqlDatabase::addDatabase(driver);
    db.setHostName(host);
    db.setDatabaseName(databaseName);
    db.setUserName(user);
    db.setPassword(pass);


    if (!db.open())
    {
        QMessageBox * box = new QMessageBox(this);
        box->setText("Cannot connect to database");
        box->setDetailedText(db.lastError().text());
        box->exec();
    }
    else
        emit accept();



}
void LoginDialog::showServerSettings()
{
    QDialog * dialog =new QDialog;
    QDialogButtonBox * buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok);
    QVBoxLayout * layout = new QVBoxLayout;
    ServerSettings * serverSettings = new ServerSettings;
    serverSettings->load();
    layout->addWidget(serverSettings);
    layout->addStretch();
    layout->addWidget(buttonBox);
    dialog->setLayout(layout);
    connect(buttonBox,SIGNAL(accepted()),dialog,SLOT(accept()));

    dialog->exec();

    serverSettings->save();
}

void LoginDialog::changeEvent(QEvent *e)
{
    QDialog::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}
